@@ -5,23 +5,35 @@ use strict;
 use warnings;
 
 use SOAP::Lite;
+use IO::Socket::SSL;
 use Net::HTTP;
 use MIME::Base64;
 use URI::Escape;
 use POSIX;
 
+# For windows
+use Encode::Locale;
+use Encode::Byte;
+
 require Exporter;
 
 our @ISA     = qw(Exporter);
-our $VERSION = '4.0.0';
+our $VERSION = '4.0.1';
 
 # ECaccess Version
-use constant VERSION => 'v4.0.0_2011221201';
+use constant VERSION => 'v4.0.1_2013040401';
 
+# Force SOAP to use the IO::Socket::SSL library!
+$ENV{'PERL_NET_HTTPS_SSL_SOCKET_CLASS'} = 'IO::Socket::SSL';
 
-# Deactivate certificate verification
+# Deactivate certificate verification in LWP (just in case)
 $ENV{'PERL_LWP_SSL_VERIFY_HOSTNAME'} = '0';
 
+# Force the SSLv3 protocol and de-activate the certificate verification
+IO::Socket::SSL::set_ctx_defaults(
+	SSL_version => 'SSLv3',
+	SSL_verify_mode => 0);
+
 # ECaccess variables
 
 my $data_channel;
@@ -36,6 +48,56 @@ my $https_ecaccess = '';
 
 sub new() {
 
+	# SOAP package to deal with reconnections!
+    BEGIN {
+      package My::SOAP::Lite;
+      use SOAP::Lite;
+      
+      @My::SOAP::Lite::ISA = 'SOAP::Lite';
+      my $retry = 0;
+      
+      sub setRetry($) {
+		$retry = $_[0];
+	  }
+
+      sub call {
+        my $self = shift;
+        return $self->SUPER::call unless @_;
+        my $result = $self->SUPER::call(@_);
+      	return $result if $self->transport->is_success;
+        my $try = $retry;
+        # If retry is requested and the error is a transport layer error!
+        while ( $try > 0 && ref($result) eq '') {
+	  	  warn "Failed connection (" . $try-- . " attempt(s) remaining)\n";
+	  	  sleep 5;
+          $result = $self->SUPER::call(@_);
+          return $result if $self->transport->is_success;
+        }
+	    # Get the error message
+	    if ( ref($result) eq '' ) {
+		  die ($self->transport->status . "\n");
+	    } else {
+		  die ($result->faultstring . "\n");
+	    }
+      }
+    }
+
+	# Get the parameters
+	my $package = shift;
+    my $self = {
+        retry => shift,
+        debug  => shift
+    };
+    
+	# Check if debug is requested?
+	if ( $self->{debug} ) {
+		eval "use SOAP::Lite +trace => 'debug';";
+		eval "use IO::Socket::SSL qw(debug3);";
+	}
+
+	# Set the SSL connection retry
+	My::SOAP::Lite::setRetry( $self->{retry} );
+
 	# Just in case the environment parameters are
 	# not set
 	no warnings qw(uninitialized);
@@ -54,39 +116,17 @@ sub new() {
 
 	# Init control channel
 	$control_channel =
-	  SOAP::Lite->uri('http://service.client.ecmwf/xsd')->proxy( 'https://' . $https_ecaccess . '/axis2/services/ECaccessService?wsdl', timeout => 5000 )
-	  ->on_fault( \&soapHandler );
+	  My::SOAP::Lite->uri('http://service.client.ecmwf/xsd')->proxy( 'https://' . $https_ecaccess . '/axis2/services/ECaccessService?wsdl', timeout => 5000 )
+	  ->on_fault( sub{} );
 
 	# Init data channel
 	$data_channel =
-	  SOAP::Lite->uri('http://service.client.ecmwf/xsd')->proxy( 'http://' . $http_ecaccess . '/axis2/services/ECaccessService?wsdl', timeout => 5000 )
-	  ->on_fault( \&soapHandler );
+	  My::SOAP::Lite->uri('http://service.client.ecmwf/xsd')->proxy( 'http://' . $http_ecaccess . '/axis2/services/ECaccessService?wsdl', timeout => 5000 )
+	  ->on_fault( sub{} );
 
-	# SOAP handler
-	sub soapHandler {
-		my $soap = shift;
-		my $res  = shift;
-
-		if ( ref($res) eq '' ) {
-			die( $soap->transport->status . "\n" );
-		}
-		else {
-			die( $res->faultstring . "\n" );
-		}
-
-		return new SOAP::SOM;
-	}
-
-	my $package = shift;
 	return bless( {}, $package );
 }
 
-sub setDebug($) {
-	if ( $_[1] ) {
-		eval "use SOAP::Lite +trace => 'debug';";
-	}
-}
-
 # Get the Control Channel (https)
 
 sub getControlChannel() {
@@ -437,7 +477,7 @@ the API:
 This Control Channel will allow you to call any method of the API. For example,
 let's move a FILE from ECfs to the super-computer:
 
-  controlChannel->moveFile($token,'ec:test/a.out','c1a:/c1a/tmp/systems/syi/a.out');
+  controlChannel->moveFile($token,'ec:test/a.out','c2a:/c2a/tmp/systems/syi/a.out');
 
 Or let's get the CosInfo from ECMWF:
 
@@ -521,7 +561,7 @@ A few examples:
   "home:/syi/bin/a.out"			a.out file in the $HOME/bin directory of user syi
   "ec:bin/a.out"			a.out file in the ECFS bin directory of the current user
   "ec:/syi/bin/a.out"			a.out file in the ECFS bin directory of user syi
-  "c1a:/c1a/tmp/systems/syi/a.out"	a.out file in the /c1a/tmp/systems/syi/ directory of c1a
+  "c2a:/c2a/tmp/systems/syi/a.out"	a.out file in the /c2a/tmp/systems/syi/ directory of c2a
 
 =head1 WHERE CAN I FIND EXAMPLES?
 
